////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2012 Kentaro Sekimoto  All rights reserved.
////////////////////////////////////////////////////////////////////////////

/*
 * FM3_I2C_driver.cpp
 *
 *  Created on: 2012/05/23
 *      Author: Kentaro Sekimoto
 */
#include <tinyhal.h>
#include "..\FM3.h"
#include "FM3_I2C_driver.h"

#pragma optimize("", off)
#pragma GCC optimization_level 0
#pragma GCC optimize ("O")

#define DEBUG_I2C

void CPU_INTC_Priority(UINT32 Irq_Index, UINT32 Irq_Pri);

BOOL m_FM3_I2C_Driver_Initialized = FALSE;
FM3_I2C_Driver g_FM3_I2C_Driver[FM3_I2C_Driver::c_MaxCh];

FM3_MFS_I2C_TypeDef *g_I2C[8] = {
    (FM3_MFS_I2C_TypeDef *)(FM3_MFS0_I2C),
    (FM3_MFS_I2C_TypeDef *)(FM3_MFS1_I2C),
    (FM3_MFS_I2C_TypeDef *)(FM3_MFS2_I2C),
    (FM3_MFS_I2C_TypeDef *)(FM3_MFS3_I2C),
    (FM3_MFS_I2C_TypeDef *)(FM3_MFS4_I2C),
    (FM3_MFS_I2C_TypeDef *)(FM3_MFS5_I2C),
    (FM3_MFS_I2C_TypeDef *)(FM3_MFS6_I2C),
    (FM3_MFS_I2C_TypeDef *)(FM3_MFS7_I2C)
};

const UINT8 I2C_SDA[] = {
    FM3__GPIO::PB5,
    FM3__GPIO::P57,        // CH1: FRAM
    FM3__GPIO::P73,
    FM3__GPIO::P76,
    FM3__GPIO::P06,
    FM3__GPIO::P37,
    FM3__GPIO::P54,
    FM3__GPIO::PB1
};

const UINT8 I2C_SCL[] = {
    FM3__GPIO::PB6,
    FM3__GPIO::P58,        // CH1: FRAM
    FM3__GPIO::P74,
    FM3__GPIO::P77,
    FM3__GPIO::P07,
    FM3__GPIO::P38,
    FM3__GPIO::P55,
    FM3__GPIO::PB2
};

#if 0
// I2C Idx
// 0   0
// 1   1
// 2   2
// 3   3
// 4   4
// 5   5
// 6   6
// 7   7
int IntIdxToI2C[8] = {0, 1, 2, 3, 4, 5, 6, 7};
int I2CToIntIdx[8] = {0, 1, 2, 3, 4, 5, 6, 7};
#endif

void WriteToSubordinate(UINT32 ch, I2C_HAL_XACTION_UNIT *unit)
{
    FM3_MFS_I2C_TypeDef *i2c = g_I2C[ch];
    UINT8 *queueData = unit->m_dataQueue.Pop();
    ASSERT(queueData);
#ifdef DEBUG_I2C
    lcd_printf("W:%02x ", *queueData);
#endif
    i2c->i2c03.TDR = *queueData;
    ++unit->m_bytesTransferred;
    --unit->m_bytesToTransfer;
}

void ReadFromSubordinate(UINT32 ch, I2C_HAL_XACTION_UNIT *unit)
{
    FM3_MFS_I2C_TypeDef *i2c = g_I2C[ch];
    UINT8* queueData;
    queueData = unit->m_dataQueue.Push();
    ASSERT(queueData);
    UINT8 data = i2c->i2c03.RDR;
#ifdef DEBUG_I2C
    lcd_printf("R:%02x ", data);
#endif
    *queueData = data;
    ++unit->m_bytesTransferred;
    --unit->m_bytesToTransfer;
}

void FM3_I2C_Handle_IRQ(UINT32 ch)
{
    INTERRUPT_START;
    UINT8 sla;
    BOOL isRead;
    FM3_MFS_I2C_TypeDef *i2c = g_I2C[ch];
    UINT8 ibcr = i2c->i2c03.IBCR;
    UINT8 ibsr = i2c->i2c03.IBSR;
    I2C_HAL_XACTION* xAction = g_FM3_I2C_Driver[ch].m_currentXAction;
    I2C_HAL_XACTION_UNIT* unit = g_FM3_I2C_Driver[ch].m_currentXActionUnit;
    isRead = unit->IsReadXActionUnit();
    size_t len = unit->m_bytesToTransfer;
    if (ibcr & IBCR_BER) {
#ifdef DEBUG_I2C
        lcd_printf("I2C ERR IBCR=%02X IBSR=%02X\r\n", ibcr, ibsr);
#endif
        ASSERT(0);
    }
    if (ibsr & IBSR_SPC) {
        i2c->i2c03.IBSR &= ~IBSR_SPC;
        i2c->i2c03.IBCR &= ~(IBCR_CNDE | IBCR_SCC);
    } else
    if (!isRead) {
            if (ibsr & IBSR_RACK) {
#ifdef DEBUG_I2C
                lcd_printf("I2C ERR TNACK IBCR=%02X IBSR=%02X\r\n", ibcr, ibsr);
#endif
//              i2c->i2c03.IBCR &= ~(IBCR_MSS | IBCR_SCC| IBCR_INTE | IBCR_INT);
//              i2c->i2c03.IBCR |= (IBCR_CNDE);
                i2c->i2c03.IBCR &= ~(IBCR_INT);
            } else {
                if (len > 0) {
                    // write
                    WriteToSubordinate(ch, unit);
                    i2c->i2c03.IBCR =       // 0x84
                            IBCR_MSS |      // select master
                            IBCR_INTE;      // enable interrupt
                } else {
                    if (!xAction->ProcessingLastUnit()) {
                        FM3_I2C_Driver::MasterXAction_Start(ch, xAction, true);
                    } else {
                        xAction->Signal(I2C_HAL_XACTION::c_Status_Completed);
                    }
                }
            }
        } else {
            // read
            if (ibsr & IBSR_RSC) {
                i2c->i2c03.IBSR &= ~IBSR_RSC;
                if (len > 1) {
                    i2c->i2c03.IBCR =       // 0xA4
                            IBCR_MSS |      // select master
                            IBCR_ACKE |     // invoke "L" as acknowledge
                            IBCR_INTE;      // enable interrupt
                } else if (len == 1) {
                    i2c->i2c03.IBCR =       // 0x84
                            IBCR_MSS |      // select master
                            IBCR_INTE;      // enable interrupt
                }
            } else if (ibsr & IBSR_RACK) {
//                i2c->i2c03.IBCR &= ~(IBCR_INT);
                if (len == 0)
                    xAction->Signal(I2C_HAL_XACTION::c_Status_Completed);
            } else {
                if (len > 1) {
                    ReadFromSubordinate(ch, unit);
                    i2c->i2c03.IBCR =       // 0xA4
                            IBCR_MSS |      // select master
                            IBCR_ACKE |     // invoke "L" as acknowledge
                            IBCR_INTE;      // enable interrupt
                } else if (len == 1) {
                    ReadFromSubordinate(ch, unit);
                    i2c->i2c03.IBCR =       // 0x84
                            IBCR_MSS |      // select master
                            IBCR_INTE;      // enable interrupt
                } else if (len == 0) {
                    xAction->Signal(I2C_HAL_XACTION::c_Status_Completed);
                }
            }
        }
    INTERRUPT_END
}

void I2C0RX_IRQHandler(void *param) __attribute__ ((interrupt));
void I2C1RX_IRQHandler(void *param) __attribute__ ((interrupt));
void I2C2RX_IRQHandler(void *param) __attribute__ ((interrupt));
void I2C3RX_IRQHandler(void *param) __attribute__ ((interrupt));
void I2C4RX_IRQHandler(void *param) __attribute__ ((interrupt));
void I2C5RX_IRQHandler(void *param) __attribute__ ((interrupt));
void I2C6RX_IRQHandler(void *param) __attribute__ ((interrupt));
void I2C7RX_IRQHandler(void *param) __attribute__ ((interrupt));
void I2C0TX_IRQHandler(void *param) __attribute__ ((interrupt));
void I2C1TX_IRQHandler(void *param) __attribute__ ((interrupt));
void I2C2TX_IRQHandler(void *param) __attribute__ ((interrupt));
void I2C3TX_IRQHandler(void *param) __attribute__ ((interrupt));
void I2C4TX_IRQHandler(void *param) __attribute__ ((interrupt));
void I2C5TX_IRQHandler(void *param) __attribute__ ((interrupt));
void I2C6TX_IRQHandler(void *param) __attribute__ ((interrupt));
void I2C7TX_IRQHandler(void *param) __attribute__ ((interrupt));

void I2C0RX_IRQHandler(void *param)
{
    FM3_I2C_Handle_IRQ(0);
}

void I2C1RX_IRQHandler(void *param)
{
    FM3_I2C_Handle_IRQ(1);
}

void I2C2RX_IRQHandler(void *param)
{
    FM3_I2C_Handle_IRQ(2);
}

void I2C3RX_IRQHandler(void *param)
{
    FM3_I2C_Handle_IRQ(3);
}

void I2C4RX_IRQHandler(void *param)
{
    FM3_I2C_Handle_IRQ(4);
}

void I2C5RX_IRQHandler(void *param)
{
    FM3_I2C_Handle_IRQ(5);
}

void I2C6RX_IRQHandler(void *param)
{
    FM3_I2C_Handle_IRQ(6);
}

void I2C7RX_IRQHandler(void *param)
{
    FM3_I2C_Handle_IRQ(7);
}

void I2C0TX_IRQHandler(void *param)
{
    FM3_I2C_Handle_IRQ(0);
}

void I2C1TX_IRQHandler(void *param)
{
    FM3_I2C_Handle_IRQ(1);
}

void I2C2TX_IRQHandler(void *param)
{
    FM3_I2C_Handle_IRQ(2);
}

void I2C3TX_IRQHandler(void *param)
{
    FM3_I2C_Handle_IRQ(3);
}

void I2C4TX_IRQHandler(void *param)
{
    FM3_I2C_Handle_IRQ(4);
}

void I2C5TX_IRQHandler(void *param)
{
    FM3_I2C_Handle_IRQ(5);
}

void I2C6TX_IRQHandler(void *param)
{
    FM3_I2C_Handle_IRQ(6);
}

void I2C7TX_IRQHandler(void *param)
{
    FM3_I2C_Handle_IRQ(7);
}

MFSTX_IRQHandler *I2CTX_IRQHandler[8] = {
    (MFSTX_IRQHandler *)I2C0TX_IRQHandler,
    (MFSTX_IRQHandler *)I2C1TX_IRQHandler,
    (MFSTX_IRQHandler *)I2C2TX_IRQHandler,
    (MFSTX_IRQHandler *)I2C3TX_IRQHandler,
    (MFSTX_IRQHandler *)I2C4TX_IRQHandler,
    (MFSTX_IRQHandler *)I2C5TX_IRQHandler,
    (MFSTX_IRQHandler *)I2C6TX_IRQHandler,
    (MFSTX_IRQHandler *)I2C7TX_IRQHandler
};

MFSRX_IRQHandler *I2CRX_IRQHandler[8] = {
    (MFSRX_IRQHandler *)I2C0RX_IRQHandler,
    (MFSRX_IRQHandler *)I2C1RX_IRQHandler,
    (MFSRX_IRQHandler *)I2C2RX_IRQHandler,
    (MFSRX_IRQHandler *)I2C3RX_IRQHandler,
    (MFSRX_IRQHandler *)I2C4RX_IRQHandler,
    (MFSRX_IRQHandler *)I2C5RX_IRQHandler,
    (MFSRX_IRQHandler *)I2C6RX_IRQHandler,
    (MFSRX_IRQHandler *)I2C7RX_IRQHandler
};

BOOL FM3_I2C_Driver::Initialize(UINT32 ch)
{
    GLOBAL_LOCK(irq);
    FM3_MFS_I2C_TypeDef *i2c = g_I2C[ch];
    if ((ch == 1) || (ch == 5) || (ch == 7)) {
        g_FM3_I2C_Driver[ch].m_initialized  = TRUE;
        g_FM3_I2C_Driver[ch].m_currentXAction = NULL;
        g_FM3_I2C_Driver[ch].m_currentXActionUnit = NULL;
        g_FM3_I2C_Driver[ch].m_state = 0;
        i2c->i2c03.ISMK = 0;    // Disable
        i2c->i2c03.SMR = 0x80;
        i2c->i2c03.SSR = 0;
        i2c->i2c03.BGR = F_PCLK/I2C_BPS - 1;
        i2c->i2c03.ISBA = 0;    // Doesn't compare slave address
        i2c->i2c03.ISMK = 0xFF; // Enable
        switch (ch) {
        case 1:
            FM3_GPIO->EPFR07 = (FM3_GPIO->EPFR07 & ~(0x0f << 12)) | (5 << 12);
            FM3_GPIO->PFR5 |= (3 << 7);        // P57, P58
            break;
        case 5:
            FM3_GPIO->EPFR08 = (FM3_GPIO->EPFR08 & ~(0x0f << 12)) | (15 << 12);
            FM3_GPIO->PFR3 |= (3 << 7);        // P37, P38
            break;
        case 7:
            FM3_GPIO->EPFR08 = (FM3_GPIO->EPFR08 & ~(0x0f << 24)) | (15 << 24);
            FM3_GPIO->PFRB |= (3 << 1);        // PB1, PB2
            break;
        }
        CPU_INTC_ActivateInterrupt(MFS0RX_IRQn + ch*2, I2CRX_IRQHandler[ch], 0);
        CPU_INTC_ActivateInterrupt(MFS0TX_IRQn + ch*2, I2CTX_IRQHandler[ch], 0);
        CPU_INTC_Priority(MFS0RX_IRQn + ch*2, 2);
        CPU_INTC_Priority(MFS0TX_IRQn + ch*2, 2);
        return TRUE;
    } else
        return FALSE;
}

BOOL FM3_I2C_Driver::Uninitialize(UINT32 ch)
{
    GLOBAL_LOCK(irq);
    if ((ch == 1) || (ch == 5) || (ch == 7)) {
        g_FM3_I2C_Driver[ch].m_initialized = FALSE;
        FM3_MFS_I2C_TypeDef *i2c = g_I2C[ch];
        //i2c->i2c03.IBCR = 0;
        i2c->i2c03.ISMK = 0;    // Disable
        CPU_INTC_DeactivateInterrupt(MFS0RX_IRQn + ch*2);
        CPU_INTC_DeactivateInterrupt(MFS0TX_IRQn + ch*2);
    }
    return TRUE;
}

void FM3_I2C_Driver::MasterXAction_Start(UINT32 ch, I2C_HAL_XACTION *xAction, bool repeatedStart)
{
    NATIVE_PROFILE_HAL_PROCESSOR_I2C();
    UINT8 sla;
    BOOL isRead;
    g_FM3_I2C_Driver[ch].m_currentXAction = xAction;
    g_FM3_I2C_Driver[ch].m_currentXActionUnit = xAction->m_xActionUnits[xAction->m_current++];
    isRead = g_FM3_I2C_Driver[ch].m_currentXActionUnit->IsReadXActionUnit();
    // ToDo: clock rate
    FM3_MFS_I2C_TypeDef *i2c = g_I2C[ch];
    sla = (xAction->m_address << 1) | (isRead ? 1:0);
#ifdef DEBUG_I2C
    if (repeatedStart)
        lcd_printf("I2C RS SLA=%02X\r\n", sla);
    else
        lcd_printf("I2C ST SLA=%02X\r\n", sla);
#endif
    i2c->i2c03.TDR = sla;
    if (repeatedStart) {
        i2c->i2c03.IBCR =   // 0xc5
                IBCR_MSS |  // select master
                IBCR_SCC |  // invoke repeated-start condition
                IBCR_INTE | // enable interrupt
                IBCR_INT;   // set interrupt flag
    } else {
        i2c->i2c03.IBCR =   //
                IBCR_MSS |  // select master
                IBCR_INTE | // enable interrupt
                IBCR_INT;   // set interrupt
    }
}

void FM3_I2C_Driver::MasterXAction_Stop(UINT32 ch)
{
    NATIVE_PROFILE_HAL_PROCESSOR_I2C();
    FM3_MFS_I2C_TypeDef *i2c = g_I2C[ch];
    // cause stop condition
    i2c->i2c03.IBCR &= ~(
            IBCR_MSS |      // clear master
            IBCR_SCC |      // clear repeated-start condition
            IBCR_INTE |     // disable interrupt
            IBCR_INT);      // clear interrupt flag
    //i2c->i2c03.IBCR |= IBCR_CNDE;
    g_FM3_I2C_Driver[ch].m_currentXAction = NULL;
    g_FM3_I2C_Driver[ch].m_currentXActionUnit = NULL;
#ifdef DEBUG_I2C
    lcd_printf("I2C Stop                     \r\n");
#endif
}

void FM3_I2C_Driver::GetClockRate(UINT32 ch, UINT32 rateKhz, UINT8& clockRate, UINT8& clockRate2)
{

}

void FM3_I2C_Driver::GetPins(UINT32 ch, GPIO_PIN& scl, GPIO_PIN& sda)
{
    NATIVE_PROFILE_HAL_PROCESSOR_I2C();
    scl = (GPIO_PIN)I2C_SCL[ch];
    sda = (GPIO_PIN)I2C_SDA[ch];
}
