////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2012 Kentaro Sekimoto  All rights reserved.
////////////////////////////////////////////////////////////////////////////
/*
 * FM3_EMAC_lwip_driver.cpp
 *
 *  Created on: 2012/03/03
 *      Author: Kentaro Sekimoto
 */
#include <tinyhal.h>
#include "net_decl_lwip.h"
#include "lwip\netif.h"
#include "lwip\tcp.h"
#include "lwip\tcpip.h"
#include "lwip\dhcp.h"
#include "lwip\pbuf.h"
#include "netif\etharp.h"
#include "..\FM3.h"
#include "FM3_EMAC_lwip.h"
#include "FM3_EMAC_lwip_driver.h"

//#define DEBUG_FM3_EMAC_LWIP_DRIVER
void CPU_INTC_Priority(UINT32 Irq_Index, UINT32 Irq_Pri);

extern NETWORK_CONFIG g_NetworkConfig;
extern int PHY_AutoNegotiate(void);
extern void ETH0_IRQHandler(void* param) __attribute__((interrupt));

struct netif g_FM3_EMAC_NetIF;

#ifndef HW_INTERRUPT
#ifdef INTERRUPT_TASK_CONTINUATION
HAL_CONTINUATION    InterruptTaskContinuation;
#else
HAL_COMPLETION      InterruptTaskCompletion;
#endif
#endif
HAL_COMPLETION      LwipUpTimeCompletion;
static BOOL         LwipNetworkStatus = 0;
static UINT32       LwipLastIpAddress = 0;

void FM3_EMAC_status_callback(struct netif *netif)
{
#ifdef DEBUG_FM3_EMAC_LWIP_DRIVER
    hal_printf("FM3_status_callback\r\n");
#endif
    if (LwipLastIpAddress != netif->ip_addr.addr) {
        Network_PostEvent(NETWORK_EVENT_TYPE_ADDRESS_CHANGED, 0);
        LwipLastIpAddress = netif->ip_addr.addr;
    }

#if defined(DEBUG)
#if 0
    lcd_printf("\f\n\n\n\n\n\nLink Update: \n");
    lcd_printf("         IP: %d.%d.%d.%d\n", (netif->ip_addr.addr >>  0) & 0xFF,
            (netif->ip_addr.addr >>  8) & 0xFF,
            (netif->ip_addr.addr >> 16) & 0xFF,
            (netif->ip_addr.addr >> 24) & 0xFF);
    lcd_printf("         GW: %d.%d.%d.%d\n", (netif->gw.addr >>  0) & 0xFF,
            (netif->gw.addr >>  8) & 0xFF,
            (netif->gw.addr >> 16) & 0xFF,
            (netif->gw.addr >> 24) & 0xFF);
#endif
    hal_printf("\nLink Update: \r\n");
    hal_printf("         IP: %d.%d.%d.%d\r\n",
            (netif->ip_addr.addr >> 0) & 0xFF,
            (netif->ip_addr.addr >> 8) & 0xFF,
            (netif->ip_addr.addr >> 16) & 0xFF,
            (netif->ip_addr.addr >> 24) & 0xFF);
    hal_printf("         GW: %d.%d.%d.%d\r\n",
            (netif->gw.addr >> 0) & 0xFF,
            (netif->gw.addr >> 8) & 0xFF,
            (netif->gw.addr >> 16) & 0xFF,
            (netif->gw.addr >> 24) & 0xFF);
#endif
}

err_t FM3_EMAC_ethhw_init(struct netif *myNetIf)
{
#ifdef DEBUG_FM3_EMAC_LWIP_DRIVER
    hal_printf("FM3_EMAC_ethhw_init\r\n");
#endif
    myNetIf->mtu = FM3_EMAC_MAX_FRAME_SIZE;
    myNetIf->output = etharp_output;
    myNetIf->linkoutput = FM3_EMAC_LWIP_xmit;
    myNetIf->status_callback = FM3_EMAC_status_callback;
    FM3_EMAC_LWIP_open(myNetIf);
    return (err_t)ERR_OK;
}

void lwip_network_uptime_completion(void *arg)
{
    NATIVE_PROFILE_PAL_NETWORK();

    BOOL status = FM3_EMAC_LWIP_GetLinkStatus();
    if (status != LwipNetworkStatus) {
        struct netif* pNetIf = (struct netif*)arg;
        if (status) {
            SOCK_NetworkConfiguration *pNetCfg = &g_NetworkConfig.NetworkInterfaces[0];
            PHY_AutoNegotiate();    // ENET_PHY_lwip_set_link_speed( );
            netif_set_up(pNetIf);
            if (pNetCfg->flags & SOCK_NETWORKCONFIGURATION_FLAGS_DHCP)
                dhcp_start(pNetIf);
            Network_PostEvent(NETWORK_EVENT_TYPE__AVAILABILITY_CHANGED, NETWORK_EVENT_FLAGS_IS_AVAILABLE);
        } else {
            netif_set_down((struct netif*)arg);
            Network_PostEvent(NETWORK_EVENT_TYPE__AVAILABILITY_CHANGED, 0);
        }
        LwipNetworkStatus = status;
    }
    LwipUpTimeCompletion.EnqueueDelta64(UPTIME_PERIOD);
}

void InitContinuations(struct netif *pNetIf)
{
#ifndef HW_INTERRUPT
#ifdef INTERRUPT_TASK_CONTINUATION
    InterruptTaskContinuation.InitializeCallback((HAL_CALLBACK_FPN)FM3_EMAC_LWIP_recv, pNetIf);
#else
    InterruptTaskCompletion.InitializeForISR((HAL_CALLBACK_FPN)FM3_EMAC_LWIP_recv, pNetIf);
    InterruptTaskCompletion.EnqueueDelta64(INT_PERIOD);
#endif
#endif
    LwipUpTimeCompletion.InitializeForUserMode((HAL_CALLBACK_FPN)lwip_network_uptime_completion, pNetIf);
    LwipUpTimeCompletion.EnqueueDelta64(UPTIME_PERIOD);
}

BOOL Network_Interface_Bind(int index)
{
#ifdef DEBUG_FM3_EMAC_LWIP_DRIVER
//    hal_printf("Network_Interface_Bind\r\n");
#endif
    return FM3_EMAC_LWIP_Driver::Bind();
}

int  Network_Interface_Open(int index)
{
#ifdef DEBUG_FM3_EMAC_LWIP_DRIVER
//    hal_printf("Network_Interface_Open\r\n");
#endif
    return FM3_EMAC_LWIP_Driver::Open(index);
}

BOOL Network_Interface_Close(int index)
{
#ifdef DEBUG_FM3_EMAC_LWIP_DRIVER
//    hal_printf("Network_Interface_Close\r\n");
#endif
    return FM3_EMAC_LWIP_Driver::Close();
}

int FM3_EMAC_LWIP_Driver::Open(int index)
{
    struct ip_addr ipaddr, subnetmask, gateway;
    struct netif *pNetIF;
    int len;
    const SOCK_NetworkConfiguration *iface;

#ifdef DEBUG_FM3_EMAC_LWIP_DRIVER
    hal_printf("FM3_EMAC_LWIP_Driver::Open\r\n");
#endif
    iface = &g_NetworkConfig.NetworkInterfaces[index];
    len = g_FM3_EMAC_NetIF.hwaddr_len;
    if (len == 0 || iface->macAddressLen < len) {
        len = iface->macAddressLen;
        g_FM3_EMAC_NetIF.hwaddr_len = len;
    }
    memcpy(g_FM3_EMAC_NetIF.hwaddr, iface->macAddressBuffer, len);
    ipaddr.addr     = iface->ipaddr;
    gateway.addr    = iface->gateway;
    subnetmask.addr = iface->subnetmask;

    g_FM3_EMAC_NetIF.flags |= NETIF_FLAG_IGMP;
    pNetIF = netif_add(&g_FM3_EMAC_NetIF, &ipaddr, &subnetmask, &gateway, NULL, FM3_EMAC_ethhw_init, ethernet_input);

#ifdef HW_INTERRUPT
#ifdef ETH_CH0
    CPU_INTC_ActivateInterrupt(ETHER_MAC0_IRQn,  ETH0_IRQHandler, 0);
    CPU_INTC_Priority(ETHER_MAC0_IRQn, FM3_ETHER_PRIORITY);
#endif
#ifdef ETH_CH1
    CPU_INTC_ActivateInterrupt(ETHER_MAC1_IRQn,  ETH1_IRQHandler, 0);
    CPU_INTC_Priority(ETHER_MAC1_IRQn, FM3_ETHER_PRIORITY);
#endif
#endif
    netif_set_default(pNetIF);
    LwipNetworkStatus = FM3_EMAC_LWIP_GetLinkStatus();
    if (LwipNetworkStatus)
        netif_set_up(pNetIF);
    InitContinuations(pNetIF);
    return g_FM3_EMAC_NetIF.num;
}

BOOL FM3_EMAC_LWIP_Driver::Close(void)
{
    int retVal = -1;
#ifdef DEBUG_FM3_EMAC_LWIP_DRIVER
    hal_printf("FM3_EMAC_LWIP_Driver::Close\r\n");
#endif
    LwipUpTimeCompletion.Abort();
#ifndef HW_INTERRUPT
#ifdef INTERRUPT_TASK_CONTINUATION
    InterruptTaskContinuation.Abort();
#else
    InterruptTaskCompletion.Abort();
#endif
#endif
    LwipNetworkStatus = 0;
    netif_set_down(&g_FM3_EMAC_NetIF);
    netif_remove(&g_FM3_EMAC_NetIF);
    FM3_EMAC_LWIP_close(&g_FM3_EMAC_NetIF);
    memset(&g_FM3_EMAC_NetIF, 0, sizeof g_FM3_EMAC_NetIF);
    return true;
}

BOOL FM3_EMAC_LWIP_Driver::Bind(void)
{
#ifdef DEBUG_FM3_EMAC_LWIP_DRIVER
    hal_printf("FM3_EMAC_LWIP_Driver::Bind\r\n");
#endif
    return true;
}

BOOL FM3_EMAC_LWIP_GetLinkStatus(void)
{
    return true;
}
