////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Microsoft Corporation.  All rights reserved.
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Portinos Copyright (c) 2012 Kentaro Sekimoto  All rights reserved.
////////////////////////////////////////////////////////////////////////////

#ifndef _TOUCH_PANEL_DRIVER_H_
#define _TOUCH_PANEL_DRIVER_H_ 1

#include <tinyhal.h>

//--//

#define FM3_TOUCHPANEL_POLL_TIMESPAN	4000	// 4ms
// CN2-01 PB0(AN16)	XIN - X-axis and Touch Check
// CN2-02 PB1(AN17) YIN - Y-axis
// CN2-33 P36		XEN
// CN2-34 P37		YEN
// CN2-35 P91		YHL
// CN2-36 P92		PD - LCD PD
// CN2-31 PF1		PDINT
#define	XEN_HI	{FM3_GPIO->DDR3 |= 0x00000040; FM3_GPIO->PDOR3 |= 0x00000040;}
#define	XEN_LO	{FM3_GPIO->DDR3 |= 0x00000040; FM3_GPIO->PDOR3 &= ~0x00000040;}
#define	YEN_HI	{FM3_GPIO->DDR3 |= 0x00000080; FM3_GPIO->PDOR3 |= 0x00000080;}
#define	YEN_LO	{FM3_GPIO->DDR3 |= 0x00000080; FM3_GPIO->PDOR3 &= ~0x00000080;}
#define	YHL_HI	{FM3_GPIO->DDR9 |= 0x00000002; FM3_GPIO->PDOR9 |= 0x00000002;}
#define	YHL_LO	{FM3_GPIO->DDR9 |= 0x00000002; FM3_GPIO->PDOR9 &= ~0x00000002;}
#define	PD_HI	{FM3_GPIO->DDR9 |= 0x00000004; FM3_GPIO->PDOR9 |= 0x00000004;}
#define	PD_LO	{FM3_GPIO->DDR9 |= 0x00000004; FM3_GPIO->PDOR9 &= ~0x00000004;}
#define	PD_IN	{FM3_GPIO->DDR9 &= ~0x00000004;}
#define	PD_HZ	{FM3_GPIO->DDR9 |= 0x00000004; FM3_GPIO->PDOR9 |= 0x00000004; FM3_GPIO->PZR9 |= 0x00000004;}
#define	PDINT_IN	{FM3_GPIO->DDRF &= ~0x00000002;}
#define	IDLE		{XEN_HI;YEN_HI;PD_HZ;}
#define	SENSE_ON	{XEN_HI;YEN_LO;YHL_LO;PD_HI;}
#define	TOUCH_CHK		(!(FM3_GPIO->PDIRB & 0x00000001))
#define	TOUCH_CHK_ON	{FM3_GPIO->ADE &= ~0x00010000; FM3_GPIO->PFRB &= ~0x00000001; FM3_GPIO->DDR3 &= ~0x00000001; FM3_GPIO->PCRB |= 0x00000001;}
#define	TOUCH_CHK_OFF	{FM3_GPIO->ADE |= 0x00010000;  FM3_GPIO->PFRB |= 0x00000001; }
#define	SENSE_Y_ON	{XEN_LO;YEN_HI;YHL_LO;PD_HZ;}
#define	SENSE_X_ON	{XEN_HI;YEN_LO;YHL_HI;PD_HZ;}
#define AD_Y_ENABLE  {FM3_GPIO->ADE |= 0x00010000;  FM3_GPIO->PFRB |= 0x00000001;}
#define AD_Y_DISABLE {FM3_GPIO->ADE &= ~0x00010000; FM3_GPIO->PFRB &= ~0x00000001;}
#define AD_X_ENABLE  {FM3_GPIO->ADE |= 0x00020000;  FM3_GPIO->PFRB |= 0x00000002;}
#define AD_X_DISABLE {FM3_GPIO->ADE &= ~0x00020000; FM3_GPIO->PFRB &= ~0x00000002;}

// default calibration values
#define X_UL	834
#define Y_UL	657
#define X_DL	880
#define Y_DL	3636
#define X_UR	3351
#define Y_UR	648
#define X_DR	3272
#define Y_DR	3587

/*
 * FM3 A-D converter
 * Register Bit Definition
 */
#define ADCR_OVRIE           0x01U
#define ADCR_CMPIE           0x02U
#define ADCR_PCIE            0x04U
#define ADCR_SCIE            0x08U
#define ADCR_CMPIF           0x20U
#define ADCR_PCIF            0x40U
#define ADCR_SCIF            0x80U
#define ADCR_IFALL           (ADCR_CMPIF | ADCR_PCIF | ADCR_SCIF)

#define ADSR_SCS             0x01U
#define ADSR_PCS             0x02U
#define ADSR_PCNS            0x04U
#define ADSR_FDAS            0x40U
#define ADSR_ADSTP           0x80U

#define SCCR_SSTR            0x01U
#define SCCR_SHEN            0x02U
#define SCCR_RPT             0x04U
#define SCCR_SFCLR           0x10U
#define SCCR_SOVR            0x20U
#define SCCR_SFUL            0x40U
#define SCCR_SEMP            0x80U
#define SCFDL_INVL           0x1000U

#define PCCR_PSTR            0x01U
#define PCCR_PHEN            0x02U
#define PCCR_PEEN            0x04U
#define PCCR_ESCE            0x08U
#define PCCR_PFCLR           0x10U
#define PCCR_POVR            0x20U
#define PCCR_PFUL            0x40U
#define PCCR_PEMP            0x80U
#define PCFDL_INVL           0x1000U

#define CMPCR_CMD0           0x20U
#define CMPCR_CMD1           0x40U
#define CMPCR_CMPEN          0x80U

#define ADCEN_ENBL           0x01U
#define ADCEN_READY          0x02U

class FM3_TouchPanel_Driver
{
public:
    static BOOL Initialize(GPIO_INTERRUPT_SERVICE_ROUTINE touchIsrProc);
    static BOOL Uninitialize();
    static HRESULT GetDeviceCaps(unsigned int iIndex, void* lpOutput);
    static void GetPoint(TOUCH_PANEL_SAMPLE_FLAGS* pTipState, int* pSource, int* pUnCalX, int* pUnCalY);
private:
    static void Poll();
    static void TouchCompletion(void* arg);
    static BOOL CalibrationPointGet(TOUCH_PANEL_CALIBRATION_POINT *pTCP);
    //
    INT32 m_global_touch;
    UINT16 m_global_inX;
    UINT16 m_global_inY;
    INT32 m_x_min;
    INT32 m_x_max;
    INT32 m_y_min;
    INT32 m_y_max;
    HAL_COMPLETION	m_touchCompletion;
    INT32 m_samplingTimespan;
};
#endif

